/*
 *    draw functions inside the pixmap
 */


#include "ct1.h"

void 
Add_atom (GtkWidget * mainw, GdkPixmap * picture, int event_x, int event_y)
/* reads the string from the text entry box and inserts it at the current
   cursor position in the current justification */
{
  struct xy_co *coord;
  gchar *name;

  int x, y;
  int i;


  name = gtk_entry_get_text (GTK_ENTRY (textbox));
  if (strlen (name))
    {
      x = round_coord (event_x, size_factor);
      y = round_coord (event_y, size_factor);
      coord = position (x, y, x, y, No_angle);
	for (i=0;i<strlen(name);i++) if (name[i]==' ') name[i]='\\';
  del_char (select_char (event_x, event_y, size_factor));
      add_char (coord->x, coord->y, name, text_direct, 0);
      modify = True;
      Display_Mol (drawing_area, picture);
	}
}

void 
Fetch_atom (GtkWidget * mainw, GdkPixmap * picture, int event_x, int event_y)
/* loads a label from the current cursor position into the text entry box */
{
  struct dc *hpc;

    hpc = select_char(event_x, event_y, size_factor);
	if (hpc) gtk_entry_set_text(GTK_ENTRY(textbox),hpc->c);
}


void 
Del_atom (GtkWidget * w, GdkPixmap * picture, int event_x, int event_y)
/* deletes the label at the current cursor position */
{
  del_char (select_char (event_x, event_y, size_factor));
  modify = True;
  Display_Mol (drawing_area, picture);
}

void 
Set_vector (GtkWidget * w, GdkPixmap * picture, int event_x, int event_y)
/* draws a circle of transient marker boxes corresponding to the current
   grid angles around the cursor position */
{
  int angle_count[] =
  {0, 12, 10, 10, 8};
  int d, dmax;
  struct xy_co *coord;

  hp.x = round_coord (event_x, size_factor);
  hp.y = round_coord (event_y, size_factor);
  coord = position (hp.x, hp.y, hp.x, hp.y, draw_angle);
  hp.x = coord->x;
  hp.y = coord->y;
  hp.tx = hp.x;
  hp.ty = hp.y;
  dmax = angle_count[draw_angle];
  for (d = 0; d < dmax; d++)
    {
      coord = calc_angle (hp.x, hp.y, draw_angle, d);
      gdk_draw_rectangle (picture, w->style->black_gc,
	TRUE, coord->x * size_factor - 2, coord->y * size_factor - 2, 4, 4);
    }
  CopyPlane ((GtkWidget *) drawing_area, (GdkPixmap *) picture);
}

void 
Add_vector (GtkWidget * w, GdkPixmap * picture, int event_x, int event_y, int cbond)
/* adds a bond vector from the previous to the current cursor position */
{
  if ((hp.x - hp.tx) || (hp.y - hp.ty))
    {
      add_struct (hp.x, hp.y, hp.tx, hp.ty, cbond, 0, 0);
    }
  Display_Mol (drawing_area, picture);
  modify = True;
}

void 
Put_vector (GtkWidget * w, GdkPixmap * picture, int event_x, int event_y)
/* draws the transient line from the previous to the current cursor position */
{
  int x, y;
  struct xy_co *coord;

  x = round_coord (event_x, size_factor);
  y = round_coord (event_y, size_factor);
  coord = position (x, y, hp.x, hp.y, draw_angle);
  x = coord->x;
  y = coord->y;
  if (x != hp.x || y != hp.y)
    {
      gdk_draw_line (picture, w->style->white_gc, hp.x * size_factor,
	      hp.y * size_factor, hp.tx * size_factor, hp.ty * size_factor);
      hp.tx = x;
      hp.ty = y;
      gdk_draw_line (picture, w->style->black_gc, hp.x * size_factor,
	      hp.y * size_factor, hp.tx * size_factor, hp.ty * size_factor);

      CopyPlane (drawing_area, picture);
    }
}

void 
Del_vector (GtkWidget * drawing_area, GdkPixmap * picture, int event_x, int event_y)
/* deletes the bond vector closest to the current cursor position */
{
  del_struct (select_vector (event_x, event_y, size_factor));
  modify = True;
  Display_Mol (drawing_area, picture);
}

void 
Invert_vector (GtkWidget * drawing_area, GdkPixmap * picture, int event_x, int event_y)
/* exchanges start and endpoint coordinates of a bond (to change direction of
   a wedge or arrow ) */
{
  struct data *hdc;
  int tmp_x, tmp_y;

  hdc = select_vector (event_x, event_y, size_factor);
  if (!hdc)
    return;
  tmp_x = hdc->x;
  tmp_y = hdc->y;
  hdc->x = hdc->tx;
  hdc->y = hdc->ty;
  hdc->tx = tmp_x;
  hdc->ty = tmp_y;
  modify = True;
  Display_Mol (drawing_area, picture);
}

void 
Add_double (GtkWidget * drawing_area, GdkPixmap * picture, int event_x, int event_y)
/* Changes bondtype :
   0 - single
   1 - leftdouble
   2 - rightdouble
   3 - triple
   4 - middouble 
   5 - wedge
   6 - dashed wedge
   7 - wiggly
   8 - single arrow
   9 - half arrow
   10 - wide single
   11 - circle 
   12 - dotted line
   13 - crossing single line
 */

{
  struct data *hdc;
  int dx, dy;

  hdc = select_vector (event_x, event_y, size_factor);
  if (hdc)
    {
      dx = hdc->x - hdc->tx;
      dy = hdc->y - hdc->ty;
      if (hdc->bond >= 13)
	hdc->bond = 0;
      else
	++hdc->bond;
      modify = True;
    }
  Display_Mol (drawing_area, picture);
}

void 
Set_start_rec (GtkWidget * w, int event_x, int event_y)
/* defines first corner of the selection rectangle in mark mode */
{
  hp.x = event_x;
  hp.y = event_y;
  hp.tx = event_x;
  hp.ty = event_y;
}

void 
Put_rec (GtkWidget * w, GdkPixmap * picture, int event_x, int event_y)
/* draws the rubberband rectangle in mark mode */
{
  gdk_draw_rectangle (picture, w->style->white_gc,
		      FALSE, hp.x, hp.y, hp.tx - hp.x, hp.ty - hp.y);

  hp.tx = event_x;
  hp.ty = event_y;
  if (hp.tx < hp.x)
    hp.tx = hp.x;
  if (hp.ty < hp.y)
    hp.ty = hp.y;
  gdk_draw_rectangle (picture, w->style->black_gc,
		      FALSE, hp.x, hp.y, hp.tx - hp.x, hp.ty - hp.y);

  CopyPlane (drawing_area, picture);
}

void 
Del_rec (GtkWidget * w, GdkPixmap * picture)
/* deletes the marked fragment inside the rectangle */
{
  if (hp.tx < hp.x)
    hp.tx = hp.x;
  if (hp.ty < hp.y)
    hp.ty = hp.y;
/*
  gdk_draw_rectangle (picture, w->style->black_gc,
		      FALSE, hp.x, hp.y, hp.tx - hp.x, hp.ty - hp.y);

  CopyPlane (drawing_area, picture);
*/
  if (hp.tx == hp.x && hp.ty == hp.y)
    {
      del_char (select_char (hp.x, hp.y, size_factor));
      del_struct (select_vector (hp.x, hp.y, size_factor));
      	FreePix (drawing_area, (GdkPixmap *) picture);
	CreatePix (drawing_area, (GdkPixmap *) picture);
	Display_Mol (drawing_area, picture);
      modify = True;
    }
  else
    {
      modify = partial_delete (round_coord (hp.x, size_factor), round_coord (hp.y, size_factor),
	round_coord (hp.tx, size_factor), round_coord (hp.ty, size_factor));
	if (modify== True) mark.flag=False;
/*	FreePix (drawing_area, picture);
      Display_Mol (drawing_area, picture);*/
      
    }
}

void 
Mark_rec (GtkWidget * w, GdkPixmap * picture, int event_x, int event_y)
/* mark all bonds and labels inside the rectangular selection area */
{
  struct data *hp_bond;
  struct dc *hp_atom;
  int hn;
  int d;
  int marks=0;

  if (hp.tx < hp.x)
    hp.tx = hp.x;
  if (hp.ty < hp.y)
    hp.ty = hp.y;
  gdk_draw_rectangle (picture, w->style->black_gc,
		      FALSE, hp.x, hp.y, hp.tx - hp.x, hp.ty - hp.y);
#ifdef LIBUNDO
  undo_snapshot ();
#endif
  CopyPlane (drawing_area, picture);
  if (hp.tx == hp.x && hp.ty == hp.y)
    {
      mark.flag = False;
    }
  else
    {
      mark.x = round_coord (hp.x, size_factor);
      mark.y = round_coord (hp.y, size_factor);
      mark.w = round_coord (hp.tx - hp.x, size_factor);
      mark.h = round_coord (hp.ty - hp.y, size_factor);
      mark.flag = True;

      hp_atom = dac_root.next;
      hn = hp.nc;
      for (d = 0; d < hn; d++)
	{
	  if (check_pos_rec (hp_atom->x, hp_atom->y, mark.x, mark.y, mark.x + mark.w, mark.y + mark.h))
	    {
	      hp_atom->marked = 1;
	      marks++;
/*fprintf(stderr,"label markiert\n"); */
	    }
	  else
	    {
	      hp_atom->marked = 0;
	    }
	  hp_atom = hp_atom->next;
	}

      hp_bond = da_root.next;
      hn = hp.n;
      for (d = 0; d < hn; d++)
	{
	  if (check_pos_rec (hp_bond->x, hp_bond->y, mark.x, mark.y, mark.x + mark.w, mark.y + mark.h))
	    {
	      hp_bond->smarked = 1;
	      marks++;
/*fprintf(stderr,"anfang markiert\n"); */
	    }
	  else
	    {
	      hp_bond->smarked = 0;
	    }
	  if (check_pos_rec (hp_bond->tx, hp_bond->ty, mark.x, mark.y, mark.x + mark.w, mark.y + mark.h))
	    {
	      hp_bond->tmarked = 1;
	      marks++;
/*fprintf(stderr,"ende markiert\n"); */
	    }
	  else
	    {
	      hp_bond->tmarked = 0;
	    }
	  hp_bond = hp_bond->next;
	}
    }
/* ??    Display_Mol(drawing_area,picture); */

/*reset mark.flag, if we did not find anything to mark */
	if (marks==0) mark.flag = False;
}

void 
Unmark_all()  /*(GtkWidget * w, GdkPixmap * picture)*/
/* remove all marks*/
{
  struct data *hp_bond;
  struct dc *hp_atom;
  int hn;
  int d;

#ifdef LIBUNDO
  undo_snapshot ();
#endif
      mark.flag =False;

      hp_atom = dac_root.next;
      hn = hp.nc;
      for (d = 0; d < hn; d++)
	{
	  hp_atom->marked = 0;
	  hp_atom = hp_atom->next;
	}

      hp_bond = da_root.next;
      hn = hp.n;
      for (d = 0; d < hn; d++)
	{
	      hp_bond->smarked = 0;
	      hp_bond->tmarked = 0;
	  hp_bond = hp_bond->next;
	}
}


void 
Mark_atom (GtkWidget * w, GdkPixmap * picture, int event_x, int event_y)
/* marks a single position (for addition of fragments) */
{
  if (hp.tx < hp.x)
    hp.tx = hp.x;
  if (hp.ty < hp.y)
    hp.ty = hp.y;

  mark.x = round_coord (hp.x, size_factor);
  mark.y = round_coord (hp.y, size_factor);
  mark.w = round_coord (hp.tx - hp.x, size_factor);
  mark.h = round_coord (hp.ty - hp.y, size_factor);
  mark.flag = True;
  addflag = 1;
  mark.flag = False;
  Display_Mol (drawing_area, picture);
}

void 
Center (GtkWidget * w)
/* callback function to center the molecule in the drawing area */
{
  FreePix (drawing_area, (GdkPixmap *) picture);
  CreatePix (drawing_area, (GdkPixmap *) picture);
  center_mol ();
  Display_Mol (drawing_area, (GdkPixmap *) picture);
  modify = True;
}

void 
Set_pmove (GtkWidget * w, int event_x, int event_y)
/*  sets the reference position for moving/rotating the selected fragment */
{
  struct xy_co *coord;
    struct data *hp_bond;
  struct dc *hp_atom;

  int x, y;
  int d,dtmp;

  x = round_coord (event_x, size_factor);
  y = round_coord (event_y, size_factor);
  coord = position (x, y, x, y, No_angle);
  hp.x = coord->x;
  hp.y = coord->y;
  xcent = hp.x;
  ycent = hp.y;

dtmp=0;
  hp_bond = da_root.next;
  for (d = 0; d < hp.n; d++)
    {

      if (hp_bond->smarked == 1)
	{
	  tmpx[dtmp] = hp_bond->x - xcent;
	  tmpy[dtmp++] = hp_bond->y - ycent;
	}
      if (hp_bond->tmarked == 1)
	{
	  tmpx[dtmp] = hp_bond->tx - xcent;
	  tmpy[dtmp++] = hp_bond->ty - ycent;
	}
      hp_bond = hp_bond->next;
    }

  hp_atom = dac_root.next;
  for (d = 0; d < hp.nc; d++)
    {
      if (hp_atom->marked == 1)
	{
	  tmpx[dtmp] = hp_atom->x - xcent;
	  tmpy[dtmp++] = hp_atom->y - ycent;
	}
      hp_atom = hp_atom->next;
    }
/*fprintf(stderr, "copied %d coordinates\n",dtmp-1);*/
}

void 
Put_pmove (GtkWidget * w, GdkPixmap * picture, int event_x, int event_y)
/* updates the position of the marked fragment as the mouse is dragged */
{
  int x, y;
  int dx, dy;
  x = round_coord (event_x, size_factor);
  y = round_coord (event_y, size_factor);

  if (mark.flag)
    {
/*    fprintf(stderr,"mark.flag ->blockmove\n"); */
/*      if (check_pos_rec (x, y, mark.x, mark.y, mark.w + mark.x, mark.y + mark.h))
*/
	{
	  dx = x - hp.x;
	  dy = y - hp.y;
	  if (partial_move (mark.x, mark.y, mark.w + mark.x,
			    mark.y + mark.h, dx, dy))
	    modify = True;
	  mark.x = mark.x + dx;
	  mark.y = mark.y + dy;
	  hp.x = x;
	  hp.y = y;
	  Display_Mol (drawing_area, picture);
	}
    }
  else
    {
      if (move_pos (hp.x, hp.y, x, y))
	{
	  hp.x = x;
	  hp.y = y;
	  Display_Mol (drawing_area, picture);
	  modify = True;
	}
    }
}

void 
Put_protate (GtkWidget * w, GdkPixmap * picture, int event_x, int event_y)
/* updates the position of the marked fragment as it pivots around the
   reference position proportional to the x movement of the mouse */
{
  int x, y;
  int dx;
static  float ang=0.;
  double rotsin, rotcos;

  x = round_coord (event_x, size_factor);
  y = round_coord (event_y, size_factor);

  if (!mark.flag)
    return;			/* rotation only makes sense for blocks */
/*
   fprintf(stderr,"mark.flag ->blockrotate\n");
 */
  mark.x = 0;
  mark.y = 0;
  mark.w = 800;
  mark.h = 800;
  dx = x - hp.x;

  ang += (float) dx / 10.;
  if (ang < -180.)
    ang = ang + 180.;
  if (ang > 180.)
    ang = ang - 180.;
/*    fprintf(stderr,"mark.flag ->blockrotate %f\n",ang); */
  rotsin = sin (ang * M_PI / 180.);
  rotcos = cos (ang * M_PI / 180.);
  if (partial_rotate (xcent, ycent, rotsin, rotcos))
    modify = True;
  hp.x = x;
  hp.y = y;
  Display_Mol (drawing_area, picture);


}

void 
Set_pscale (GtkWidget * w, int event_x, int event_y)
/*  sets the reference position for moving/rotating the selected fragment */
{
  struct xy_co *coord;
    struct data *hp_bond;
  struct dc *hp_atom;

  int x, y;
  int d,dtmp;

  x = round_coord (event_x, size_factor);
  y = round_coord (event_y, size_factor);
  coord = position (x, y, x, y, No_angle);
  hp.x = coord->x;
  hp.y = coord->y;
  xcent = hp.x;
  ycent = hp.y;

dtmp=0;
  hp_bond = da_root.next;
  for (d = 0; d < hp.n; d++)
    {

      if (hp_bond->smarked == 1)
	{
	  tmpx[dtmp] = hp_bond->x - xcent;
	  tmpy[dtmp++] = hp_bond->y - ycent;
	}
      if (hp_bond->tmarked == 1)
	{
	  tmpx[dtmp] = hp_bond->tx - xcent;
	  tmpy[dtmp++] = hp_bond->ty - ycent;
	}
      hp_bond = hp_bond->next;
    }

  hp_atom = dac_root.next;
  for (d = 0; d < hp.nc; d++)
    {
      if (hp_atom->marked == 1)
	{
	  tmpx[dtmp] = hp_atom->x - xcent;
	  tmpy[dtmp++] = hp_atom->y - ycent;
	}
      hp_atom = hp_atom->next;
    }
/*fprintf(stderr, "copied %d coordinates\n",dtmp-1);*/
}

void 
Put_pscale (GtkWidget * w, GdkPixmap * picture, int event_x, int event_y)
/* updates the position of the marked fragment as the mouse is dragged */
{
  int x;
  int dx;
  x = round_coord (event_x, size_factor);

  if (mark.flag)
    {
	{
	  dx = x - hp.x;
	  if (partial_rescale (mark.x, mark.y, mark.w + mark.x,
			    mark.y + mark.h, dx))
	    modify = True;
	  Display_Mol (drawing_area, picture);
	}
    }
}
